/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * switchcase.c
 * This file is part of gedit
 *
 * Copyright (C) 2002 Paolo Borelli 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glade/glade-xml.h>
#include <libgnome/gnome-i18n.h>
#include <libgnome/gnome-help.h>

#include <gedit-plugin.h>
#include <gedit-debug.h>
#include <gedit-menus.h>
#include <gedit-utils.h>

#define MENU_ITEM_LABEL	N_("Change Case")
#define MENU_ITEM_PATH	"/menu/Edit/EditOps_6/"
#define MENU_ITEM_NAME	"ChangeCasePlugin"	
#define MENU_ITEM_TIP	N_("Change case of selected text")


G_MODULE_EXPORT GeditPluginState update_ui (GeditPlugin *plugin, BonoboWindow *window);
G_MODULE_EXPORT GeditPluginState init (GeditPlugin *pd);
G_MODULE_EXPORT GeditPluginState destroy (GeditPlugin *pd);
G_MODULE_EXPORT GeditPluginState activate (GeditPlugin *pd);
G_MODULE_EXPORT GeditPluginState deactivate (GeditPlugin *pd);


typedef struct {
	GtkWidget *dialog;
	GtkWidget *rb_upper;
	GtkWidget *rb_lower;
	GtkWidget *rb_switch;
} ChangeCaseDialog;


typedef enum {
	TO_UPPER_CASE,
	TO_LOWER_CASE,
	SWITCH_CASE,
} ChangeCaseChoice;


static gchar*
do_upper_case (const gchar *text)
{
	static gchar *ret_text;

	ret_text = g_utf8_strup (text, -1);
	return ret_text;	
}


static gchar*
do_lower_case (const gchar *text)
{
	static gchar *ret_text;

	ret_text = g_utf8_strdown (text, -1);
	return ret_text;	
}


static gchar*
do_switch_case (const gchar *text)
{
	const gchar *p = text;
	static gchar *ret_text;
	GString *s = NULL;

	s = g_string_new ("\0");

	while (*p) {
		gunichar c = g_utf8_get_char (p);
		gunichar nc;

		if (g_unichar_islower (c))
			nc = g_unichar_toupper (c);
		else
			nc = g_unichar_tolower (c);

		s = g_string_append_unichar (s, nc);
		
		p = g_utf8_next_char (p);
	}
	
	ret_text = s->str;

	g_string_free (s, FALSE);

	return ret_text;	
}


static void
change_case (ChangeCaseChoice choice)
{
	GeditDocument *doc;
	GeditView *view;
	gint start;
	gint end;
	gchar *buffer;
	gchar *buffer_utf8;
	gchar *changed_text;

	gedit_debug (DEBUG_PLUGINS, "");

	view = gedit_get_active_view ();
	g_return_if_fail (view != NULL);

	doc = gedit_view_get_document (view);
	g_return_if_fail (doc != NULL);

	if (!gedit_document_get_selection (doc, &start, &end))
		/* Should warn that no text is selected? */
		return;
	
	buffer = gedit_document_get_chars (doc, start, end);
	
	if (g_utf8_validate (buffer, -1, NULL))
		buffer_utf8 = buffer;
	else {
		buffer_utf8 = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
		g_free (buffer);
	}

	switch (choice) {
		case TO_UPPER_CASE:
			changed_text = do_upper_case (buffer_utf8);
			break;

		case TO_LOWER_CASE:
			changed_text = do_lower_case (buffer_utf8);
			break;

		case SWITCH_CASE:
			changed_text = do_switch_case (buffer_utf8);
			break;

		default:
			/* Should never get here */
			g_return_if_reached();

	}

	/* not sure if this is raelly undoable */
	gedit_document_begin_not_undoable_action (doc);

	gedit_document_delete_text (doc, start, end);
	gedit_document_set_cursor (doc, start);
	gedit_document_insert_text_at_cursor (doc, changed_text, -1);

	gedit_document_end_not_undoable_action (doc);

	g_free (buffer_utf8);
	g_free (changed_text);

}


static void
dialog_destroyed (GtkObject *obj,  void **dialog_pointer)
{
	gedit_debug (DEBUG_PLUGINS, "");

	if (dialog_pointer != NULL)
	{
		g_free (*dialog_pointer);
		*dialog_pointer = NULL;
	}	
}


static void
dialog_response_handler (GtkDialog *dlg, gint res_id, ChangeCaseDialog *dialog)
{
	GError *error = NULL;

	gedit_debug (DEBUG_PLUGINS, "");

	switch (res_id) {
		case GTK_RESPONSE_APPLY:
			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->rb_upper)))
				change_case (TO_UPPER_CASE);
			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->rb_lower)))
				change_case (TO_LOWER_CASE);
			if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (dialog->rb_switch)))
				change_case (SWITCH_CASE);
			gtk_widget_destroy (dialog->dialog);
			break;

		case GTK_RESPONSE_HELP:
			/* FIXME: Choose a better link id */
			gnome_help_display ("gedit.xml", "gedit-use-plugins", &error);
	
			if (error != NULL)
			{
				g_warning (error->message);
				g_error_free (error);
			}
			break;
			
		default:
			gtk_widget_destroy (GTK_WIDGET (dialog->dialog));
	}
}


static ChangeCaseDialog*
change_case_dialog_create (void)
{
	static ChangeCaseDialog *dialog = NULL;

	GladeXML  *gui;
	GtkWindow *window;
	GtkWidget *content;

	gedit_debug (DEBUG_PLUGINS, "");

	window = GTK_WINDOW (gedit_get_active_window ());

	if (dialog != NULL)
	{
		gtk_window_present (GTK_WINDOW (dialog->dialog));
		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), window);
		return dialog;
	}

	gui = glade_xml_new (GEDIT_GLADEDIR "changecase.glade2",
	                     "changecase_dialog_content", NULL);

	if (!gui) {
		g_warning ("Could not find changecase.glade2, reinstall gedit.\n");
		return NULL;
	}
	
	dialog = g_new0 (ChangeCaseDialog, 1);

	dialog->dialog = gtk_dialog_new_with_buttons (_("Change Case"),
	                                             window,
	                                             GTK_DIALOG_DESTROY_WITH_PARENT,
	                                             GTK_STOCK_CANCEL,
	                                             GTK_RESPONSE_CANCEL,
	                                             GTK_STOCK_APPLY,
	                                             GTK_RESPONSE_APPLY,
	                                             GTK_STOCK_HELP,
	                                             GTK_RESPONSE_HELP,
	                                             NULL);

	g_return_val_if_fail (dialog->dialog != NULL, NULL);

	content = glade_xml_get_widget (gui, "changecase_dialog_content");
	g_return_val_if_fail (content != NULL, NULL);

	dialog->rb_upper = glade_xml_get_widget (gui, "radiobtn_upper");
	g_return_val_if_fail (dialog->rb_upper != NULL, NULL);

	dialog->rb_lower = glade_xml_get_widget (gui, "radiobtn_lower");
	g_return_val_if_fail (dialog->rb_lower != NULL, NULL);

	dialog->rb_switch = glade_xml_get_widget (gui, "radiobtn_switch");
	g_return_val_if_fail (dialog->rb_switch != NULL, NULL);

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->dialog)->vbox),
	                    content, FALSE, FALSE, 0);

	gtk_dialog_set_default_response (GTK_DIALOG (dialog->dialog),
	                                 GTK_RESPONSE_APPLY);

	g_signal_connect (G_OBJECT (dialog->dialog), "destroy",
			   G_CALLBACK (dialog_destroyed), &dialog);

	g_signal_connect (G_OBJECT (dialog->dialog), "response",
	                  G_CALLBACK (dialog_response_handler), dialog);

	g_object_unref (gui);

	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);

	gtk_widget_show (dialog->dialog);

	return dialog;
}


static void
change_case_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
{
	ChangeCaseDialog *dialog = NULL;

	dialog = change_case_dialog_create();
	if (dialog == NULL) 
	{
		g_warning ("Could not create Insert Table dialog");
		return;
	}
}


G_MODULE_EXPORT GeditPluginState
update_ui (GeditPlugin *plugin, BonoboWindow *window)
{
	BonoboUIComponent *uic;
	
	gedit_debug (DEBUG_PLUGINS, "");	
	g_return_val_if_fail (window != NULL, PLUGIN_ERROR);

	uic = gedit_get_ui_component_from_window (window);

	gedit_menus_set_verb_sensitive (uic, "/commands/" MENU_ITEM_NAME, TRUE);

	return PLUGIN_OK;
}


G_MODULE_EXPORT GeditPluginState
init (GeditPlugin *pd)
{
	/* initialize */
	gedit_debug (DEBUG_PLUGINS, "");
		
	return PLUGIN_OK;
}


G_MODULE_EXPORT GeditPluginState
destroy (GeditPlugin *plugin)
{
	gedit_debug (DEBUG_PLUGINS, "");

	return PLUGIN_OK;
}


G_MODULE_EXPORT GeditPluginState
activate (GeditPlugin *pd)
{
	GList *top_windows;
	gedit_debug (DEBUG_PLUGINS, "");

	top_windows = gedit_get_top_windows ();
	g_return_val_if_fail (top_windows != NULL, PLUGIN_ERROR);

	while (top_windows)
	{
		gedit_menus_add_menu_item (BONOBO_WINDOW (top_windows->data),
		                           MENU_ITEM_PATH, MENU_ITEM_NAME,
		                           MENU_ITEM_LABEL, MENU_ITEM_TIP, NULL,
		                           change_case_cb);

		pd->update_ui (pd, BONOBO_WINDOW (top_windows->data));

		top_windows = g_list_next (top_windows);
	}

	return PLUGIN_OK;
}


G_MODULE_EXPORT GeditPluginState
deactivate (GeditPlugin *pd)
{
	gedit_menus_remove_menu_item_all (MENU_ITEM_PATH, MENU_ITEM_NAME);

	return PLUGIN_OK;
}

